@@ -1032,10 +1032,20 @@ gst_validate_utils_get_clocktime (GstStructure * structure, const gchar * name,
   return TRUE;
 }
 
+/**
+ * gst_validate_object_set_property_full:
+ * @reporter: The #GstValidateReporter to use to report errors
+ * @object: The #GObject to set the property on
+ * @property: The name of the property to set
+ * @value: The value to set the property to
+ * @flags: The #GstValidateObjectSetPropertyFlags to use
+ *
+ * Since: 1.24
+ */
 GstValidateActionReturn
-gst_validate_object_set_property (GstValidateReporter * reporter,
+gst_validate_object_set_property_full (GstValidateReporter * reporter,
     GObject * object, const gchar * property,
-    const GValue * value, gboolean optional)
+    const GValue * value, GstValidateObjectSetPropertyFlags flags)
 {
   GParamSpec *paramspec;
   GObjectClass *klass = G_OBJECT_GET_CLASS (object);
@@ -1044,7 +1054,7 @@ gst_validate_object_set_property (GstValidateReporter * reporter,
 
   paramspec = g_object_class_find_property (klass, property);
   if (paramspec == NULL) {
-    if (optional)
+    if (!!(flags & GST_VALIDATE_OBJECT_SET_PROPERTY_FLAGS_OPTIONAL))
       return TRUE;
     GST_ERROR ("Target doesn't have property %s", property);
     return FALSE;
@@ -1078,16 +1088,18 @@ gst_validate_object_set_property (GstValidateReporter * reporter,
   g_value_init (&nvalue, paramspec->value_type);
   g_object_get_property (object, property, &nvalue);
 
-  if (gst_value_compare (&cvalue, &nvalue) != GST_VALUE_EQUAL) {
-    gchar *nvalstr = gst_value_serialize (&nvalue);
-    gchar *cvalstr = gst_value_serialize (&cvalue);
-    GST_VALIDATE_REPORT (reporter, SCENARIO_ACTION_EXECUTION_ERROR,
-        "Setting value %" GST_PTR_FORMAT "::%s failed, expected value: %s"
-        " value after setting %s", object, property, cvalstr, nvalstr);
+  if (!(flags & GST_VALIDATE_OBJECT_SET_PROPERTY_FLAGS_NO_VALUE_CHECK)) {
+    if (gst_value_compare (&cvalue, &nvalue) != GST_VALUE_EQUAL) {
+      gchar *nvalstr = gst_value_serialize (&nvalue);
+      gchar *cvalstr = gst_value_serialize (&cvalue);
+      GST_VALIDATE_REPORT (reporter, SCENARIO_ACTION_EXECUTION_ERROR,
+          "Setting value %" GST_PTR_FORMAT "::%s failed, expected value: %s"
+          " value after setting %s", object, property, cvalstr, nvalstr);
 
-    res = GST_VALIDATE_EXECUTE_ACTION_ERROR_REPORTED;
-    g_free (nvalstr);
-    g_free (cvalstr);
+      res = GST_VALIDATE_EXECUTE_ACTION_ERROR_REPORTED;
+      g_free (nvalstr);
+      g_free (cvalstr);
+    }
   }
 
   g_value_reset (&cvalue);
@@ -1104,6 +1116,7 @@ fault_restore (void)
   memset (&action, 0, sizeof (action));
   action.sa_handler = SIG_DFL;
 
+  sigaction (SIGINT, &action, NULL);
   sigaction (SIGSEGV, &action, NULL);
   sigaction (SIGQUIT, &action, NULL);
 }
@@ -1254,6 +1267,10 @@ gst_validate_replace_variables_in_string (gpointer source,
       }
 
       if (!var_value) {
+        g_free (varname);
+        g_free (pvarname);
+        g_free (string);
+        g_clear_pointer (&match_info, g_match_info_free);
         if (!(flags & GST_VALIDATE_STRUCTURE_RESOLVE_VARIABLES_NO_FAILURE)) {
           gst_validate_error_structure (source,
               "Trying to use undefined variable `%s`.\n"
@@ -1263,6 +1280,7 @@ gst_validate_replace_variables_in_string (gpointer source,
               varname, gst_structure_to_string (local_vars),
               (flags & GST_VALIDATE_STRUCTURE_RESOLVE_VARIABLES_LOCAL_ONLY) ?
               ": unused" : gst_structure_to_string (global_vars));
+
         }
 
         return NULL;
@@ -1354,6 +1372,19 @@ _structure_set_variables (GQuark field_id, GValue * value, ReplaceData * data)
     return TRUE;
   }
 
+
+  if (GST_VALUE_HOLDS_STRUCTURE (value)) {
+    GstStructure *s = gst_structure_copy (gst_value_get_structure (value));
+
+    gst_validate_structure_resolve_variables (data->source,
+        s, data->local_vars, data->flags);
+
+    gst_value_set_structure (value, s);
+    gst_structure_free (s);
+
+    return TRUE;
+  }
+
   if (!G_VALUE_HOLDS_STRING (value))
     return TRUE;
 
@@ -1500,7 +1531,7 @@ gst_validate_structure_set_variables_from_struct_file (GstStructure * vars,
   gchar *t, *config_name_dir;
   gchar *validateflow, *expectations_dir, *actual_result_dir;
   const gchar *logdir;
-  gboolean local = ! !vars;
+  gboolean local = !!vars;
 
   if (!struct_file)
     return;
@@ -1592,3 +1623,12 @@ gst_validate_fail_on_missing_plugin (void)
   }
   return FALSE;
 }
+
+GstValidateActionReturn
+gst_validate_object_set_property (GstValidateReporter * reporter,
+    GObject * object,
+    const gchar * property, const GValue * value, gboolean optional)
+{
+  return gst_validate_object_set_property_full (reporter, object, property,
+      value, optional ? GST_VALIDATE_OBJECT_SET_PROPERTY_FLAGS_OPTIONAL : 0);
+}
